use std::collections::BTreeMap;

use crate::{
    models::item::{Item, ItemFlag, LocationActionType},
    regular_tasks::{
        queued_command::{queue_command_for_npc, QueueCommand},
        TaskRunContext,
    },
    static_content::possession_type::{
        possession_data, ContainerFlag, PossessionData, PossessionType,
    },
    DResult,
};

pub async fn npc_combat_ai_attacking(
    ctx: &mut TaskRunContext<'_>,
    npc: &mut Item,
    _opponent: &Item,
) -> DResult<()> {
    if !npc.flags.contains(&ItemFlag::EnableCombatAi) {
        return Ok(());
    }

    // Look through the inventory and find the weapons.
    let possessed_items = ctx.trans.find_items_by_location(&npc.refstr()).await?;

    // We always truncate the queue to 1 and re-insert actions...
    npc.queue.truncate(1);

    let all_weapons = rank_weapons_for_ai(&possessed_items);
    let mut ammo_loaded_cache: BTreeMap<String, usize> = BTreeMap::new();
    queue_ai_wield_weapon(
        &all_weapons,
        ctx,
        &mut ammo_loaded_cache,
        &possessed_items,
        npc,
    )
    .await?;

    queue_weapon_reloads_for_ai(&possessed_items, all_weapons, ammo_loaded_cache, ctx, npc).await?;

    Ok(())
}

async fn queue_weapon_reloads_for_ai(
    possessed_items: &Vec<std::sync::Arc<Item>>,
    all_weapons: Vec<(String, &PossessionData)>,
    mut ammo_loaded_cache: BTreeMap<String, usize>,
    ctx: &mut TaskRunContext<'_>,
    npc: &mut Item,
) -> DResult<()> {
    let direct_ammo: Vec<(String, PossessionType)> = find_direct_ammo_for_ai(&possessed_items);
    let clips: Vec<(String, PossessionType)> = find_clips_for_ai(possessed_items);
    for (poss_id, poss_data) in &all_weapons {
        match poss_data.container_data.as_ref() {
            None => break,
            Some(cont_data) => {
                // If a weapon is loaded sufficiently, don't use more ammo...
                let ammolevel = match ammo_loaded_cache.get(poss_id.as_str()) {
                    Some(v) => *v,
                    None => {
                        let loaded_ammo = ctx
                            .trans
                            .find_items_by_location(&format!("possession/{}", &poss_id))
                            .await?;
                        let ammolevel = loaded_ammo.len();
                        ammo_loaded_cache.insert(poss_id.clone(), ammolevel);
                        ammolevel
                    }
                };
                if ammolevel > 3 {
                    break;
                }
                let mut to_load = 3 - ammolevel;

                // Check for an ammo item in inventory directly.
                queue_direct_load_ammo_for_ai(
                    &direct_ammo,
                    cont_data,
                    ammolevel,
                    &mut to_load,
                    npc,
                    ctx,
                    poss_id,
                )
                .await?;

                // We might need to load some ammo from a clip...
                queue_load_ammo_from_clip_for_ai(&clips, to_load, cont_data, ctx, npc, poss_id)
                    .await?;
            }
        }
    }
    Ok(())
}

async fn queue_load_ammo_from_clip_for_ai(
    clips: &Vec<(String, PossessionType)>,
    mut to_load: usize,
    cont_data: &crate::static_content::possession_type::ContainerData,
    ctx: &mut TaskRunContext<'_>,
    npc: &mut Item,
    poss_id: &String,
) -> DResult<()> {
    for (clip_id, ammo_type) in clips {
        if to_load == 0 {
            break;
        }
        if !cont_data.checker.check_place_type(&Some(ammo_type.clone())) {
            continue;
        }
        let clip_contents = ctx
            .trans
            .find_items_by_location(&format!("possession/{}", &clip_id))
            .await?;
        for ammo_it in &clip_contents {
            if to_load == 0 {
                return Ok(());
            }
            if ammo_it.possession_type.as_ref() != Some(ammo_type) {
                continue;
            }
            to_load -= 1;
            queue_command_for_npc(
                &ctx.trans,
                npc,
                &QueueCommand::GetFromContainer {
                    from_item_id: format!("possession/{}", clip_id),
                    get_possession_id: ammo_it.item_code.clone(),
                },
            )
            .await?;
            queue_command_for_npc(
                &ctx.trans,
                npc,
                &QueueCommand::Put {
                    container_possession_id: poss_id.clone(),
                    target_possession_id: ammo_it.item_code.clone(),
                },
            )
            .await?;
        }
    }
    Ok(())
}

async fn queue_direct_load_ammo_for_ai(
    direct_ammo: &Vec<(String, PossessionType)>,
    cont_data: &crate::static_content::possession_type::ContainerData,
    ammolevel: usize,
    to_load: &mut usize,
    npc: &mut Item,
    ctx: &mut TaskRunContext<'_>,
    poss_id: &String,
) -> DResult<()> {
    let load_ammo: Vec<String> = direct_ammo
        .iter()
        .filter(|(_, pt)| cont_data.checker.check_place_type(&Some(pt.clone())))
        .map(|(it_id, _)| it_id.clone())
        .take(ammolevel)
        .collect();
    *to_load -= load_ammo.len();
    for load_id in load_ammo {
        match npc.queue.front() {
            Some(QueueCommand::Put {
                target_possession_id,
                ..
            }) if target_possession_id == &load_id => break,
            _ => {}
        }
        queue_command_for_npc(
            &ctx.trans,
            npc,
            &QueueCommand::Put {
                container_possession_id: poss_id.clone(),
                target_possession_id: load_id.clone(),
            },
        )
        .await?;
    }
    Ok(())
}

fn find_clips_for_ai(possessed_items: &Vec<std::sync::Arc<Item>>) -> Vec<(String, PossessionType)> {
    possessed_items
        .iter()
        .filter_map(|it| {
            it.possession_type
                .as_ref()
                .and_then(|pt| possession_data().get(pt))
                .and_then(|pd| {
                    pd.container_data.as_ref().and_then(|cd| {
                        if cd.container_flags.contains(&ContainerFlag::AmmoClip) {
                            cd.default_contents
                                .first()
                                .map(|pt_in| (it.item_code.clone(), pt_in.clone()))
                        } else {
                            None
                        }
                    })
                })
        })
        .collect()
}

fn find_direct_ammo_for_ai(
    possessed_items: &Vec<std::sync::Arc<Item>>,
) -> Vec<(String, PossessionType)> {
    possessed_items
        .iter()
        .filter_map(|it| {
            match it
                .possession_type
                .as_ref()
                .and_then(|pt| possession_data().get(pt).map(|pd| (pt, pd)))
            {
                Some((pt, pd)) if pd.ammo_data.is_some() => {
                    Some((it.item_code.clone(), pt.clone()))
                }
                _ => None,
            }
        })
        .collect()
}

async fn queue_ai_wield_weapon(
    all_weapons: &Vec<(String, &PossessionData)>,
    ctx: &mut TaskRunContext<'_>,
    ammo_loaded_cache: &mut BTreeMap<String, usize>,
    possessed_items: &Vec<std::sync::Arc<Item>>,
    npc: &mut Item,
) -> DResult<()> {
    let mut best_ready_weapon: Option<String> = None;
    for (poss_code, poss_data) in all_weapons {
        if poss_data.container_data.is_none() {
            best_ready_weapon = Some(poss_code.clone());
            break;
        }
        // Check loaded status...
        let loaded_ammo = ctx
            .trans
            .find_items_by_location(&format!("possession/{}", &poss_code))
            .await?;
        let ammolevel = loaded_ammo.len();
        ammo_loaded_cache.insert(poss_code.clone(), ammolevel);
        if ammolevel > 0 {
            best_ready_weapon = Some(poss_code.clone());
            break;
        }
    }
    let current_wield = possessed_items
        .iter()
        .find(|it| it.action_type == LocationActionType::Wielded)
        .map(|it| it.item_code.clone());
    let pending_wield = npc.queue.front().and_then(|comm| match comm {
        QueueCommand::Wield { possession_id } => Some(possession_id.clone()),
        _ => None,
    });
    let eventual_wield = pending_wield.or(current_wield);
    Ok(match (eventual_wield, best_ready_weapon) {
        (_, None) => {} // Do nothing if nothing possible.
        (Some(wielded), Some(to_wield)) if wielded == to_wield => {}
        (_, Some(to_wield)) => {
            queue_command_for_npc(
                &ctx.trans,
                npc,
                &QueueCommand::Wield {
                    possession_id: to_wield.clone(),
                },
            )
            .await?
        }
    })
}

fn rank_weapons_for_ai(
    possessed_items: &Vec<std::sync::Arc<Item>>,
) -> Vec<(String, &PossessionData)> {
    let mut all_weapons: Vec<(String, &'static PossessionData)> = possessed_items
        .iter()
        .filter_map(|it| {
            match it
                .possession_type
                .as_ref()
                .and_then(|pt| possession_data().get(pt))
            {
                None => None,
                Some(pd) if pd.weapon_data.is_none() => None,
                Some(pd) => Some((it.item_code.clone(), *pd)),
            }
        })
        .collect();

    all_weapons.sort_by(|(_, pd1), (_, pd2)| {
        pd1.container_data
            .is_some()
            .cmp(&pd2.container_data.is_some())
            .then(
                pd1.weapon_data
                    .as_ref()
                    .map_or(0.0, |wd| wd.normal_attack.mean_damage)
                    .partial_cmp(
                        &pd2.weapon_data
                            .as_ref()
                            .map_or(0.0, |wd| wd.normal_attack.mean_damage),
                    )
                    .unwrap_or(std::cmp::Ordering::Equal),
            )
            .reverse()
    });
    all_weapons
}
