use super::NPC;
use crate::{
    models::{
        consent::ConsentType,
        item::{Pronouns, SkillType},
    },
    static_content::{npc::KillBonus, possession_type::PossessionType, species::SpeciesType},
};
use serde::Deserialize;
use serde_yaml::from_str as from_yaml_str;

#[derive(Deserialize)]
enum NorthradNPC {
    Scorpion {
        code: String,
        name: String,
        spawn_loc: String,
    },
    Snake {
        code: String,
        name: String,
        spawn_loc: String,
    },
}

pub fn npc_list() -> Vec<NPC> {
    from_yaml_str::<Vec<NorthradNPC>>(include_str!("northrad_npcs.yaml"))
        .unwrap()
        .into_iter()
        .map(|c| match c {
            NorthradNPC::Scorpion { code, name, spawn_loc } =>
              NPC {
                  code: format!("northrad_{}", &code),
                  name,
                  pronouns: Pronouns::default_inanimate(),
                  aliases: vec!["scorpion".to_owned()],
                  aggression: 14,
                  description: "A large scorpion. Its eyes, black like beads of obsidian, give it a menacing look. Its curved tail writhes menacingly over its back. At the tip of its tail is a sharp stinger; a drop of some kind of liquid pools on the tip".to_owned(),
                  spawn_location: format!("room/northrad_{}", &spawn_loc),
                  intrinsic_weapon: Some(PossessionType::Stinger),
                  kill_bonus: Some(KillBonus {
                      msg: "On your wristpad: The Rangelands Service thanks you for your assistance! Please accept this small reward.",
                      payment: 250,
                  }),
                  aggro_pc_only: true,
                  total_xp: 15000,
                  total_skills: SkillType::values()
                      .into_iter()
                      .map(|sk| {
                          (
                              sk.clone(),
                              match sk {
                                  SkillType::Dodge => 14.0,
                                  SkillType::Fists => 18.0,
                                  _ => 8.0
                              }
                          )
                      }).collect(),
                  player_consents: vec!(ConsentType::Fight),
                  species: SpeciesType::Scorpion,
                  message_handler: None,
                  wander_zones: vec!["northern_radfields".to_owned()],
                  ..Default::default()
              },
            NorthradNPC::Snake { code, name, spawn_loc } =>
                NPC {
                    code: format!("northrad_{}", &code),
                    name,
                    pronouns: Pronouns::default_inanimate(),
                    aliases: vec!["snake".to_owned()],
                    aggression: 14,
                    description: "A large snake, its scales glinting with a metallic sheen. Its eyes, cold and calculating, remain ever alert to potential threats. It has adopted an S-shaped posture, as if preparing to strike. It occasionally samples the air by flicking its forked tongue out of its mouth".to_owned(),
                    spawn_location: format!("room/northrad_{}", &spawn_loc),
                    intrinsic_weapon: Some(PossessionType::VenomousFangs),
                    species: SpeciesType::Snake,
                    kill_bonus: Some(KillBonus {
                        msg: "On your wristpad: The Rangelands Service thanks you for your assistance! Please accept this small reward.",
                        payment: 250,
                    }),
                    aggro_pc_only: true,
                    total_xp: 15000,
                    total_skills: SkillType::values()
                        .into_iter()
                        .map(|sk| {
                            (
                                sk.clone(),
                                match sk {
                                    SkillType::Dodge => 14.0,
                                    SkillType::Fists => 19.0,
                                    _ => 8.0
                                }
                            )
                        })
                        .collect(),
                    player_consents: vec!(ConsentType::Fight),
                    message_handler: None,
                    wander_zones: vec!["northern_radfields".to_owned()],
                    ..Default::default()
                },
        }
        ).collect()
}
