use super::{PossessionData, PossessionType, WeaponAttackData};
use crate::{
    models::item::SkillType,
    static_content::possession_type::{
        AllowlistContainerCheck, ContainerData, ContainerFlag, DamageType, SkillScaling, WeaponData,
    },
};
use once_cell::sync::OnceCell;

pub fn airsoft_round_checker() -> &'static AllowlistContainerCheck {
    static C: OnceCell<AllowlistContainerCheck> = OnceCell::new();
    &C.get_or_init(|| AllowlistContainerCheck {
        allowed_types: vec![PossessionType::AirsoftRound],
    })
}

pub fn ninemil_bullet_checker() -> &'static AllowlistContainerCheck {
    static C: OnceCell<AllowlistContainerCheck> = OnceCell::new();
    &C.get_or_init(|| AllowlistContainerCheck {
        allowed_types: vec![PossessionType::NinemilSolidBullet],
    })
}

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| {
        vec![
            (
                PossessionType::AirsoftRound,
                PossessionData {
                     display: "airsoft round",
                     details:
                         "A soft plastic spherical projectile",
                     aliases: vec!["air-soft round", "air soft round", "ammo"],
                     weight: 8,
                     ammo_data: Some(WeaponAttackData {
                         start_messages: vec![Box::new(|attacker, victim| {
                             format!(
                                 "{} points {} colourful plastic gun at {}",
                                 &attacker.display_for_sentence(1, true),
                                 &attacker.pronouns.possessive,
                                 &victim.display_for_sentence(1, false),
                             )
                         })],
                         success_messages: vec![Box::new(|attacker, victim, part| {
                             format!(
                                 "{} hits {}'s {} with an airsoft round",
                                 &attacker.display_for_sentence(1, true),
                                 &victim.display_for_sentence(1, false),
                                 &part.display(victim.sex.clone())
                             )
                         })],
                         mean_damage: 1.0,
                         stdev_damage: 0.5,
                         base_damage_type: DamageType::Bullet,
                         ..Default::default()
                     }),
                     ..Default::default()
                 },
             ),
             (
                 PossessionType::AirsoftRoundFiftyBox,
                 PossessionData {
                     display: "box of airsoft rounds",
                     details: "A box of airsoft rounds, each a soft plastic spherical projectile",
                     aliases: vec!["air-soft round box", "air soft round box", "ammo"],
                     weight: 361,
                     container_data: Some(ContainerData {
                         max_weight: 400,
                         base_weight: 1,
                         compression_ratio: 0.9,
                         default_contents: vec![PossessionType::AirsoftRound].repeat(50),
                         checker: airsoft_round_checker(),
                         container_flags: vec![ContainerFlag::DestroyOnEmpty, ContainerFlag::AmmoClip]
                     }),
                     ..Default::default()
                 }
             ),
             (
                 PossessionType::AirsoftGun,
                 PossessionData {
                     display: "airsoft gun",
                     details: "A colourful looking pistol-type weapon. Apart from the colour scheme, it is a close replica of a normal powder-powered pistol",
                     aliases: vec!["gun", "air soft gun", "air-soft gun"],
                     weapon_data: Some(
                         WeaponData {
                             uses_skill: SkillType::Pistols,
                             raw_min_to_learn: 0.0,
                             raw_max_to_learn: 2.0,
                             normal_attack: WeaponAttackData {
                                 start_messages: vec![Box::new(|attacker, victim| {
                                     format!(
                                         "{} points {} colourful light-looking gun at {}",
                                        &attacker.display_for_sentence(1, true),
                                        &attacker.pronouns.possessive,
                                        &victim.display_for_sentence(1, false),
                                    )
                                 })],
                                 success_messages: vec![Box::new(|attacker, victim, part| {
                                     format!(
                                         "There is a click as {} aims {} colourful unloaded gun at {} and pulls the trigger",
                                         &attacker.display_for_sentence(1, true),
                                         &victim.display_for_sentence(1, false),
                                         &part.display(victim.sex.clone())
                                     )
                                 })],
                                 mean_damage: 0.0,
                                 stdev_damage: 0.0,
                                 base_damage_type: DamageType::Bullet,
                                 ..Default::default()
                             },
                             power_attack: None
                         }
                     ),
                     weight: 500,
                     container_data: Some(ContainerData {
                         max_weight: 96,
                         base_weight: 500,
                         compression_ratio: 1.0,
                         checker: airsoft_round_checker(),
                         ..Default::default()
                     }),
                     ..Default::default()
                 }
             ),
            (
                PossessionType::NinemilSolidBullet,
                PossessionData {
                     display: "9mm solid bullet",
                     details:
                         "A nine millimetre bullet consisting of casing and a solid-tipped projectile",
                     aliases: vec!["bullet", "nine-millimetre bullet", "ammo"],
                     weight: 8,
                     ammo_data: Some(WeaponAttackData {
                         start_messages: vec![Box::new(|attacker, victim| {
                             format!(
                                 "{} points {} gun at {}",
                                 &attacker.display_for_sentence(1, true),
                                 &attacker.pronouns.possessive,
                                 &victim.display_for_sentence(1, false),
                             )
                         })],
                         success_messages: vec![Box::new(|attacker, victim, part| {
                             format!(
                                 "Oww! {} blasts a 9mm hole through {}'s {}",
                                 &attacker.display_for_sentence(1, true),
                                 &victim.display_for_sentence(1, false),
                                 &part.display(victim.sex.clone())
                             )
                         })],
                         mean_damage: 15.0,
                         stdev_damage: 15.0,
                         skill_scaling: vec![SkillScaling {
                             skill: SkillType::Pistols,
                             min_skill: 2.0,
                             mean_damage_per_point_over_min: 2.0,
                         }],
                         base_damage_type: DamageType::Bullet,
                         ..Default::default()
                     }),
                     ..Default::default()
                 },
             ),
             (
                 PossessionType::NinemilFiftyBox,
                 PossessionData {
                     display: "box of 9mm solid bullets",
                     details: "A box of nine millimetre bullets, each consisting of casing and a solid-tipped projectile",
                     aliases: vec!["bullet", "nine-millimetre bullet", "ammo"],
                     weight: 361,
                     container_data: Some(ContainerData {
                         max_weight: 400,
                         base_weight: 1,
                         compression_ratio: 0.9,
                         default_contents: vec![PossessionType::NinemilSolidBullet].repeat(50),
                         checker: ninemil_bullet_checker(),
                         container_flags: vec![ContainerFlag::DestroyOnEmpty, ContainerFlag::AmmoClip]
                     }),
                     ..Default::default()
                 }
             ),
             (
                 PossessionType::Z3000Pistol,
                 PossessionData {
                     display: "Z3000 pistol",
                     details: "A black metal pistol, featuring a cylindrical barrel, with a rectangular body and butt",
                     aliases: vec!["gun", "pistol"],
                     weight: 500,
                     weapon_data: Some(
                         WeaponData {
                             uses_skill: SkillType::Pistols,
                             raw_min_to_learn: 2.0,
                             raw_max_to_learn: 5.0,
                             normal_attack: WeaponAttackData {
                                 start_messages: vec![Box::new(|attacker, victim| {
                                     format!(
                                         "{} points {} light-looking gun at {}",
                                        &attacker.display_for_sentence(1, true),
                                        &attacker.pronouns.possessive,
                                        &victim.display_for_sentence(1, false),
                                    )
                                 })],
                                 success_messages: vec![Box::new(|attacker, victim, part| {
                                     format!(
                                         "There is a click as {} aims {} unloaded gun at {} and pulls the trigger",
                                         &attacker.display_for_sentence(1, true),
                                         &victim.display_for_sentence(1, false),
                                         &part.display(victim.sex.clone())
                                     )
                                 })],
                                 mean_damage: 0.0,
                                 stdev_damage: 0.0,
                                 base_damage_type: DamageType::Bullet,
                                 ..Default::default()
                             },
                             power_attack: None
                         }
                     ),
                     container_data: Some(ContainerData {
                         max_weight: 96,
                         base_weight: 500,
                         compression_ratio: 1.0,
                         checker: ninemil_bullet_checker(),
                         ..Default::default()
                     }),
                     ..Default::default()
                 }
             ),
        ]
    })
}
